(function($) {
    // this structure is the only thing that needs to be modified to override more or different fields
    var fields = [{
            "internalName": "SalesDivision",
            "parentColumnInternal": "SalesRegion",
            "childColumnInternal": "SalesDivision",
            "relationshipList": "SalesDivision",
            "relationshipListParentColumn": "SalesRegion",
            "relationshipListChildColumn": "Title",
            "debug": true
        },
        {
            "internalName": "SalesState",
            "parentColumnInternal": "SalesDivision",
            "childColumnInternal": "SalesState",
            "relationshipList": "SalesState",
            "relationshipListParentColumn": "SalesDivision",
            "relationshipListChildColumn": "Title",
            "debug": true
        }
    ];

    // helper because IE doesn't have Array.find
    function arrayFind(array, value, equals) {
        for (var i = 0; i < array.length; i++) {
            if (equals(value, array[i])) return array[i];
        }
    }

    // class instance that encapsulates CSR functionality for cascading lookups.
    var cascadDropdownCSR = {
        // this will be the render method we pass in overrides
        render: function(ctx) {
            // get the form context so we can register some callbacks
            var formCtx = SPClientTemplates.Utility.GetFormContextForCurrentField(ctx);

            // get the OOB default rendering
            var html = cascadDropdownCSR.getDefaultRendering(ctx);

            // register an init callback for our field
            formCtx.registerInitCallback(formCtx.fieldName, function() {
                // find the current config element
                var config = arrayFind(fields, formCtx.fieldName, function(v, o) {
                    return v === o.internalName;
                });

                // get the list schema for all fields
                var schema = window[ctx.FormUniqueId + "FormCtx"].ListSchema;

                // get the display names SPServices wants from the internal names
                // in our configuration
                if(config.parentColumnInternal) {
                    config.parentColumn = schema[config.parentColumnInternal].Title;
                }
                if(config.childColumnInternal) {
                    config.childColumn = schema[config.childColumnInternal].Title;
                }
                
                // if relationshipWebURL is not specified, assume the root web
                if(!config.relationshipWebURL) {
                    config.relationshipWebURL = _spPageContextInfo.siteServerRelativeUrl;
                }

                // apply SPCascadeDropdowns to our configuration
                $().SPServices.SPCascadeDropdowns(config);
            });

            // need to do get value callback because lookup updates it's value in an onchange
            // handler declared in init callback, which we've overridden
            formCtx.registerGetValueCallback(formCtx.fieldName, function() {
                var val = $("select[id^='" + formCtx.fieldName + "_']").val();
                var text = $("select[id^='" + formCtx.fieldName + "_'] option:selected").text();
                return (val == '0' || val == '' ? '' : val + ";#" + text);
            });

            // return the OOB default rendering
            return html;
        },

        getDefaultRendering: function(ctx) {
            var templatesByType = SPClientTemplates._defaultTemplates.Fields.default.all.all;
            var currentTemplates = templatesByType[ctx.CurrentFieldSchema.Type];
            var currentRenderFunc = currentTemplates[ctx.BaseViewID];
            return currentRenderFunc(ctx);
        }
    };

    /*
     * Create an empty overrides object.
     */
    var overrides = {
        Templates: {
            'Fields': {}
        }
    };

    /*
     * Add an overrides object for each field we want to customize.
     */
    for (var i = 0; i < fields.length; i++) {
        var current = fields[i];
        overrides.Templates.Fields[current.internalName] = {
            'NewForm': cascadDropdownCSR.render,
            'EditForm': cascadDropdownCSR.render,
        };
    }

    // also register templates now for non-MDS and full page loads
    SPClientTemplates.TemplateManager.RegisterTemplateOverrides(overrides);

    // register template overrides for partial page loads if MDS is enabled
    if (typeof _spPageContextInfo != 'undefined' && _spPageContextInfo != null) {
        var base = _spPageContextInfo.siteServerRelativeUrl.toLowerCase();
        var url = (base === '/' ? '' : base) + '/style%20library/cascadingdropdownscsr.js';

        RegisterModuleInit(url.toLowerCase(), function() {
            SPClientTemplates.TemplateManager.RegisterTemplateOverrides(overrides);
        });
    }
})(jQuery);